@@ -8,6 +8,9 @@ import matplotlib.pyplot as plt
 from scipy.optimize import curve_fit
 import os, collections
 import time
+from iminuit import Minuit
+from bfit.fitting.leastsquares import LeastSquares
+import warnings
 
 __doc__=\
 """
@@ -60,15 +63,22 @@ class global_fitter(object):
             
             fn                      list of fitting function handles
             fixed                   list of fixed variables (corresponds to input)
+            fprime_dx               x spacing in calculating centered differences derivative
             
             metadata                array of additional inputs, fixed for each data set
                                     (if len(shared) < len(actual inputs))
             
+            minuit                  Minuit object for minimizing with migrad algorithm
+            minimizer               string: one of "curve_fit" or "migrad"
+            
             npar                    number of parameters in input function
             nsets                   number of data sets
             
             par                     fit results with unnecessary variables stripped 
             par_runwise             fit results run-by-run with all needed inputs
+            std_l, std_u            lower/upper errors with unnecessary variables stripped 
+            std_l_runwise           lower errors run-by-run with all needed inputs
+            std_u_runwise           upper errors run-by-run with all needed inputs
             cov                     fit covarince matrix with unnecessary variables stripped
             cov_runwise             fit covarince matrix run-by-run with all needed inputs
             
@@ -78,10 +88,16 @@ class global_fitter(object):
             xcat                    concatenated xdata for global fitting
             ycat                    concatenated ydata for global fitting
             dycat                   concatenated dydata for global fitting
+            dxcat                   concatenated dydata for global fitting
+            dycat_low               concatenated dydata for global fitting
+            dxcat_low               concatenated dydata for global fitting
             
             x                       input array of x data sets [array1,array2,...]
             y                       input array of y data sets [array1,array2,...]
             dy                      input array of y error sets [array1,array2,...]
+            dx                      input array of x error sets [array1,array2,...]
+            dy_low                  input array of y error sets [array1,array2,...]
+            dx_low                  input array of x error sets [array1,array2,...]
     """
     
     # class variables
@@ -89,17 +105,23 @@ class global_fitter(object):
     ndraw_pts = 500             # number of points to draw fits with
     
     # ======================================================================= #
-    def __init__(self,x,y,dy,fn,shared,fixed=None,metadata=None):
+    def __init__(self, fn, x, y, dy=None, dx=None, dy_low=None, dx_low=None, 
+                shared=None, fixed=None, metadata=None, fprime_dx=1e-6):
         """
-            x,y:        2-list of data sets of equal length. 
-                        fmt: [[a1,a2,...],[b1,b2,...],...]
-            
-            dy:         list of errors in y with same format as y
-            
             fn:         function handle OR list of function handles. 
                         MUST specify inputs explicitly if list must have that 
                         len(fn) = len(x) and all function must have the same 
                         inputs in the same order
+        
+            x, y:       2-list of data sets of equal length. 
+                        fmt: [[a1,a2,...],[b1,b2,...],...]
+            
+            dx, dy:     list of errors in x or y with same format as x and y
+            dx_low, dy_low: list of low bound errors in x or y with same format 
+                        as x and y. 
+                            
+                        If not None, then assume dy and dx are the upper errors
+                        If None, assume symmetric errors
             
             shared:     tuple of booleans indicating which values to share. 
                         len = number of parameters 
@@ -116,34 +138,75 @@ class global_fitter(object):
                         number of parameters is set by len(shared), all 
                         remaining inputs are expected to be metadata inputs
                         function call: fn[i](x[i],*par[i],*metadata[i])
+            
+            fprime_dx:  x spacing in calculating centered differences derivative
+            
         """
         # ---------------------------------------------------------------------
         # Check and assign inputs
-        
-        # data types
+        self.fprime_dx = fprime_dx
+    
+        if (dy is None and dy_low is not None) or (dx is None and dx_low is not None): 
+            raise RuntimeError("If specifying lower errors, must also specify dy or dx")
+            
+        # data types: check and assign
+        msg = 'Lengths of input data arrays do not match.\nnsets: %s, %s =  %d, %d\n'
         x = list(x)
         y = list(y)
-        dy = list(dy)
+        
+        if not len(x) == len(y):
+            raise RuntimeError(msg % ('x', 'y', len(x), len(y)))
+        
+        self.nsets = len(x)
+        
+        if dy is not None:          
+            dy = list(dy)
+            if not self.nsets == len(dy):
+                raise RuntimeError(msg % ('n', 'dy', self.nsets, len(dy)))
+        
+        if dx is not None:          
+            dx = list(dx)
+            if not self.nsets == len(dx):
+                raise RuntimeError(msg % ('n', 'dx', self.nsets, len(dx)))
+            
+        if dy_low is not None:      
+            dy_low = list(dy_low)
+            if not self.nsets == len(dy_low):
+                raise RuntimeError(msg % ('n', 'dy_low', self.nsets, len(dy_low)))
+        
+        if dx_low is not None:      
+            dx_low = list(dx_low)
+            if not self.nsets == len(dx_low):
+                raise RuntimeError(msg % ('n', 'dx_low', self.nsets, len(dx_low)))
         
         # shared parameters
         self.shared = np.asarray(shared)
         
         # get number of input parameters
+        if shared is None:
+            raise RuntimeError("Must specified shared parameter as boolean iterable")
         self.npar = len(shared)
-        
-        # test number of data sets
-        if not len(x) == len(y) == len(dy):
-            raise RuntimeError('Lengths of input data arrays do not match.\n'+\
-                'nsets: x, y, dy =  %d, %d, %d\n' % (len(x),len(y),len(dy)))            
-        self.nsets = len(x)
-        
-        # remove points with zero error from data 
-        for i,(xdat,ydat,dydat) in enumerate(zip(x,y,dy)):
-            idx = dydat != 0
-            x[i] = xdat[idx]
-            y[i] = ydat[idx]
-            dy[i] = dydat[idx]
-        
+
+        # if errors, remove points with zero error from data
+        if dy is not None or dx is not None:
+            for i in range(self.nsets):
+                
+                # get indexing
+                idx = np.full(len(x[i]), False)
+                if dy is not None:      idx += dy[i] != 0
+                if dx is not None:      idx += dx[i] != 0
+                if dy_low is not None:  idx += dy_low[i] != 0
+                if dx_low is not None:  idx += dx_low[i] != 0
+                
+                # crop
+                x[i] = x[i][idx]
+                y[i] = y[i][idx]
+                
+                if dy is not None:      dy[i] = dy[i][idx]
+                if dx is not None:      dx[i] = dx[i][idx]
+                if dy_low is not None:  dy_low[i] = dy_low[i][idx]
+                if dx_low is not None:  dx_low[i] = dx_low[i][idx]
+
         # check if list of functions given 
         if not isinstance(fn,collections.Iterable):
             fn = [fn]*self.nsets
@@ -174,7 +237,7 @@ class global_fitter(object):
         
         fixed_flat = np.concatenate(fixed)
         
-                # check that no shared parameters are fixed
+        # check that no shared parameters are fixed
         shared_as_int = self.shared.astype(int)
         for fix in fixed: 
             if any(fix+shared_as_int>1):
@@ -215,14 +278,100 @@ class global_fitter(object):
         self.x = x
         self.y = y
         self.dy = dy
+        self.dx = dx
+        self.dy_low = dy_low
+        self.dx_low = dx_low
         self.fixed = fixed
         self.sharing_links = sharing_links
         
         # get concatenated data
         self.xcat = np.concatenate(x)
         self.ycat = np.concatenate(y)
-        self.dycat = np.concatenate(dy)
+        
+        if dy is not None:          self.dycat = np.concatenate(dy)
+        else:                       self.dycat = None
+        
+        if dx is not None:          self.dxcat = np.concatenate(dx)
+        else:                       self.dxcat = None
+        
+        if dy_low is not None:      self.dycat_low = np.concatenate(dy_low)
+        else:                       self.dycat_low = None
+        
+        if dx_low is not None:      self.dxcat_low = np.concatenate(dx_low)
+        else:                       self.dxcat_low = None
+        
+    # ======================================================================= #
+    def _do_curve_fit(self, master_fn, p0_first, **fitargs):
+        """
+            Run curve_fit minimmizer
+        """
+        
+        dycat = self.dycat
+        absolute_sigma = self.dycat is not None
             
+        if self.dxcat is not None:
+            warnings.warn("curve_fit minimizer does not account for x errors")
+        
+        par,cov = curve_fit(master_fn,
+                            self.xcat,
+                            self.ycat,
+                            sigma=dycat,
+                            absolute_sigma=absolute_sigma,
+                            p0 = p0_first,
+                            **fitargs)
+                            
+        std = np.diag(cov)**0.5
+        return (par, std, std, cov)
+    
+    # ======================================================================= #
+    def _do_migrad(self, master_fn, master_fnprime, do_minos, p0_first, **fitargs):
+        
+        # get least squares
+        ls = LeastSquares(master_fn, self.xcat, self.ycat, 
+                            dy = self.dycat, 
+                            dx = self.dxcat, 
+                            dy_low = self.dycat_low, 
+                            dx_low = self.dxcat_low,
+                            fn_prime = master_fnprime)
+        
+        # set args
+        limit = fitargs.get('bounds',None)
+        if limit is not None:
+            limit = np.array(limit).T
+        
+        kwargs_minuit = {'start':p0_first,
+                         'limit':limit,
+                         'print_level':fitargs.get('print_level',1),
+                         'errordef':1,
+                        }
+        
+        # get minuit obj
+        m = Minuit.from_array_func(ls, **kwargs_minuit)
+        self.ls = ls
+        self.minuit = m
+        
+        # minimize
+        m.migrad()
+        
+        # check minimum
+        if not m.fmin.is_valid:
+            raise RuntimeError('Minuit failed to converge to a valid minimum')
+        
+        # get errors
+        if do_minos:
+            m.minos()
+            lower, upper = m.np_merrors()
+        else:
+            m.hesse()
+            lower = m.np_errors()
+            upper = lower
+        
+        # get output
+        par = m.np_values()
+        cov = m.np_covariance()
+        
+        return (par, lower, upper, cov)
+        
     # ======================================================================= #
     def draw(self,mode='stack',xlabel='',ylabel='',do_legend=False,labels=None,
              savefig='',**errorbar_args):
@@ -261,10 +410,23 @@ class global_fitter(object):
         for i in range(self.nsets):
             
             # get data
-            x,y,dy = self.x[i], self.y[i], self.dy[i]
+            x, y, = self.x[i], self.y[i]
+            
+            if self.dy is not None:         dy = self.dy[i]
+            else:                           dy = None
+            
+            if self.dx is not None:         dx = self.dx[i]
+            else:                           dx = None
+            
+            if self.dy_low is not None:     dy_low = self.dy_low[i]
+            else:                           dy_low = dy
+            
+            if self.dx_low is not None:     dx_low = self.dx_low[i]
+            else:                           dx_low = dx
+            
             f = self.fn[i]
             md = self.metadata[i]
-            
+                        
             # make new figure
             if mode in ['new','n']:            
                 fig_list.append(plt.figure())
@@ -279,7 +441,14 @@ class global_fitter(object):
                 x_draw = x
                 
             # draw data
-            datplt = plt.errorbar(x_draw,y,dy,label=labels[i],**errorbar_args)
+            if dy is None:      dy_draw = None
+            else:               dy_draw = (dy_low, dy)    
+            
+            if dx is None:      dx_draw = None
+            else:               dx_draw = (dx_low, dx)    
+            
+            datplt = plt.errorbar(x_draw, y, yerr=dy_draw, xerr=dx_draw, 
+                                  label=labels[i], **errorbar_args)
             
             # get color for fit curve
             if mode in ['stack','s']:
@@ -288,9 +457,9 @@ class global_fitter(object):
                 color = 'k'
             
             # draw fit
-            xfit = np.linspace(min(x),max(x),self.ndraw_pts)
-            xdraw = np.linspace(min(x_draw),max(x_draw),self.ndraw_pts)
-            plt.plot(xdraw,f(xfit,*self.par_runwise[i],*md),color=color,zorder=10)
+            xfit = np.linspace(min(x), max(x), self.ndraw_pts)
+            xdraw = np.linspace(min(x_draw), max(x_draw), self.ndraw_pts)
+            plt.plot(xdraw,f(xfit, *self.par_runwise[i], *md), color=color, zorder=10)
         
             # plot elements
             plt.ylabel(ylabel)
@@ -304,7 +473,7 @@ class global_fitter(object):
         return fig_list
         
     # ======================================================================= #
-    def fit(self,**fitargs):
+    def fit(self, minimizer='migrad', do_minos=True, **fitargs):
         """
             fitargs: parameters to pass to fitter (scipy.optimize.curve_fit) 
             
@@ -331,8 +500,12 @@ class global_fitter(object):
                             
                             bounds.shape = (2,npars)
                             
-                            
-            returns (parameters,covariance matrix)
+            minimizer:      string. One of "trf", "dogbox", or "migrad" indicating 
+                            which code to use to minimize the function
+            
+            do_minos:       if true, and if minimizer==migrad, then run minos errors
+            
+            returns (parameters, lower errors, upper errors, covariance matrix)
         """
         
         # get values from self
@@ -340,6 +513,7 @@ class global_fitter(object):
         shared = self.shared
         sharing_links = self.sharing_links
         fn = self.fn
+        self.minimizer = minimizer
         
         # set default p0
         if 'p0' in fitargs:
@@ -348,10 +522,10 @@ class global_fitter(object):
             
             # expand p0
             if len(p0.shape) == 1:
-                p0 = np.full((self.nsets,self.npar),p0)
+                p0 = np.full((self.nsets, self.npar), p0)
                 
         else:
-            p0 = np.ones((self.nsets,self.npar))
+            p0 = np.ones((self.nsets, self.npar))
         
         # for fixed parameters
         p0_flat_inv = np.concatenate(p0)[::-1]
@@ -390,34 +564,56 @@ class global_fitter(object):
         x = self.x
         rng = range(self.nsets)
         metadata = self.metadata
-        def master_fn(x_unused,*par):            
-            inputs = np.take(np.hstack((par,p0_flat_inv)),sharing_links)
-            return np.concatenate([fn[i](x[i],*inputs[i],*metadata[i]) for i in rng])
+          
+        def master_fn(x_unused,*par):
+            inputs = np.take(np.hstack((par, p0_flat_inv)), sharing_links)
+            return np.concatenate([fn[i](x[i], *inputs[i], *metadata[i]) for i in rng])
         
         self.master_fn = master_fn
-          
-        # do fit
-        par,cov = curve_fit(master_fn,
-                            self.xcat,
-                            self.ycat,
-                            sigma=self.dycat,
-                            absolute_sigma=True,
-                            p0 = p0_first,
-                            **fitargs)                    
+            
+        # make derivative of master function
+        def master_fnprime(x_unused, *par):
+            
+            inputs = np.take(np.hstack((par, p0_flat_inv)), sharing_links)
+            xhi = np.concatenate([fn[i](x[i]+fprime_dx/2, *inputs[i], *metadata[i]) for i in rng])
+            xlo = np.concatenate([fn[i](x[i]-fprime_dx/2, *inputs[i], *metadata[i]) for i in rng])
+            return (xhi-xlo)/fprime_dx
+        self.master_fnprime = master_fnprime
+      
+        # do curve_fit
+        if minimizer in ('trf','dogbox'):
+            fitargs['method'] = minimizer
+            par, std_l, std_u, cov = self._do_curve_fit(master_fn, p0_first, **fitargs)
+        
+        # do migrad
+        elif minimizer == 'migrad':
+            fprime_dx = self.fprime_dx
+            self.master_fn = master_fn
+                    
+            par, std_l, std_u, cov = self._do_migrad(master_fn, master_fnprime,
+                                                    do_minos, p0_first, **fitargs)
+        else:
+            raise RuntimeError("Unrecognized minimizer input '%s'" % minimizer)
         
         # to array
         par = np.asarray(par)
+        std_l = np.asarray(std_l)
+        std_u = np.asarray(std_u)
         cov = np.asarray(cov)
         
         # inflate parameters
         par_out = np.hstack((par,p0_flat_inv))[sharing_links]
         
+        zero = np.zeros(len(p0_flat_inv))
+        std_l_out = np.hstack((std_l, zero))[sharing_links]
+        std_u_out = np.hstack((std_u, zero))[sharing_links]
+        
         # inflate covariance matrix
         cov_out = []
         for lnk in sharing_links:
             
             # init
-            cov_run = np.zeros((self.npar,self.npar))
+            cov_run = np.zeros((self.npar, self.npar))
             
             # assign
             for i in range(self.npar):
@@ -434,11 +630,15 @@ class global_fitter(object):
                     
         # return
         self.par = par
+        self.std_l = std_l
+        self.std_u = std_u
         self.cov = cov
         self.par_runwise = par_out
+        self.std_l_runwise = std_l_out
+        self.std_u_runwise = std_u_out
         self.cov_runwise = cov_out
         
-        return (par_out,cov_out)
+        return (par_out, std_l_out, std_u_out, cov_out)
     
     # ======================================================================= #
     def get_chi(self):
@@ -452,31 +652,63 @@ class global_fitter(object):
 
         # global
         dof = len(self.xcat)-len(self.par)
-        self.chi_glbl = np.sum(np.square((self.ycat-\
-                      self.master_fn(self.xcat,*self.par))/self.dycat))/dof
+        
+        if self.minimizer == 'migrad':
+            self.chi_glbl = self.minuit.fval/dof
+        else:
+            
+            ls = LeastSquares(self.master_fn, self.xcat, self.ycat, 
+                            dy = self.dycat, 
+                            dx = self.dxcat, 
+                            dy_low = self.dycat_low, 
+                            dx_low = self.dxcat_low,
+                            fn_prime = self.master_fnprime)
+            
+            self.chi_glbl = ls(self.par) / dof
             
         # single fn chisq
         self.chi = []
-        for x,y,dy,p,f,fx,m in zip(self.x,self.y,self.dy,self.par_runwise,self.fn,self.fixed,self.metadata):
-            dof = len(x)-self.npar+sum(fx)
-            self.chi.append(np.sum(np.square((y-f(x,*p,*m))/dy))/dof)
+        for i in range(self.nsets):
+            
+            x = self.x[i]
+            y = self.y[i]
+            p = self.par_runwise[i]
+            f = self.fn[i]
+            fx = self.fixed[i]
+            m = self.metadata[i]
+            
+            dy = None if self.dy is None else self.dy[i]
+            dx = None if self.dx is None else self.dx[i]
+            dy_low = None if self.dy_low is None else self.dy_low[i]
+            dx_low = None if self.dx_low is None else self.dx_low[i]
         
-        return (self.chi_glbl,self.chi)
+            ls = LeastSquares(  f, x, y, 
+                                dy = dy, 
+                                dx = dx,
+                                dx_low = dx_low,
+                                dy_low = dy_low,
+                            )
+            dof = len(x)-self.npar+sum(fx)
+            self.chi.append(ls(np.concatenate((p, m))) / dof)
+        
+        return (self.chi_glbl, self.chi)
 
     # ======================================================================= #
     def get_par(self):
         """
             Fetch fit parameters
             
-            return 2-tuple of (par,cov) with format
+            return 4-tuple of (par, std_l, std_u, cov) with format
             
-            ([data1:[par1,par2,...],data2:[],...],
-             [data1:[cov1,cov2,...],data2:[],...])
+            ([data1:[par1, par2, ...], data2:[], ...],
+             [data1:[std_l1, std_l2, ...], data2:[], ...],
+             [data1:[std_u1, std_u2, ...], data2:[], ...],
+             [data1:[cov1, cov2, ...], data2:[], ...])
         """
-        cov = self.cov_runwise
-        std = np.array(list(map(np.diag,cov)))**0.5
-        
-        return (self.par_runwise,cov,std)
+        return (self.par_runwise, 
+                self.std_l_runwise, 
+                self.std_u_runwise, 
+                self.cov_runwise)
     
     # ======================================================================= #
     def _expand_bound_lim(self,lim):
