@@ -18,16 +18,14 @@ except ImportError as errmsg:
     print('No 3D axes drawing available')
     print(errmsg)
 
-import sys,os,datetime,textwrap
+import sys, os, datetime, textwrap
+import webbrowser, subprocess, importlib, logging, warnings
 import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
 import bdata as bd
 import weakref as wref
-import webbrowser
-import subprocess
-import importlib
-import logging
+
 
 from bfit import __version__,logger_name,icon_path
 from bfit.gui.tab_fileviewer import fileviewer
@@ -43,11 +41,16 @@ from bfit.gui.popup_set_ppm_reference import popup_set_ppm_reference
 from bfit.gui.popup_set_histograms import popup_set_histograms
 from bfit.backend.PltTracker import PltTracker
 from bfit.backend.raise_window import raise_window
+from bfit.backend.fitdata import fitdata
 import bfit.backend.colors as colors
 
 # interactive plotting
 plt.ion()
 
+# filter warnings related to new dkeys on read
+warnings.simplefilter('ignore',RuntimeWarning)
+    
+
 __doc__="""
     BNMR/BNQR data visualization and curve fitting.
     
@@ -100,6 +103,7 @@ class bfit(object):
             logger:         logging object 
             logger_name:    string of unique logger name
             mainframe:      main frame for the object
+            minimizer:      StringVar: path to python module with fitter object
             notebook:       contains all tabs for operations:
                 fileviewer
                 fetch_files
@@ -185,7 +189,7 @@ class bfit(object):
                  }
     
     # valid thermometer channels to read from
-    thermo_keys = ('A','B','(A+B)/2')
+    thermo_keys = ('A', 'B', '(A+B)/2')
     
     # draw axis labels
     xlabel_dict={'20':"Time (s)",
@@ -216,22 +220,28 @@ class bfit(object):
            '1n':'mV'}
     
     # units: mode:[conversion rate from original to display units,unit]
-    units = {'1f':(1e-6,'MHz'),
-             '2e':(1e-6,'MHz'),
-             '1w':(1,'Hz'),
-             '1n':(1e-3,'V')}
+    units = {'1f':[1e-6,'MHz'],
+             '2e':[1e-6,'MHz'],
+             '1w':[1,'Hz'],
+             '1n':[1e-3,'V']}
+    
+    # minimizers
+    minimizers = {'curve_fit (trf)':'bfit.fitting.fitter_curve_fit', 
+                  'migrad (hesse)':'bfit.fitting.fitter_migrad_hesse',
+                  'migrad (minos)':'bfit.fitting.fitter_migrad_minos',
+                  }
     
     data = {}   # for fitdata objects
     
     # define draw componeents in draw_param and labels
-    draw_components = ['Temperature (K)','1000/T (1/K)','Impl. Energy (keV)',
+    draw_components = ('Temperature (K)','1000/T (1/K)','Impl. Energy (keV)',
                        'Platform Bias (kV)','Run Number','B0 Field (T)', 
                        'Unique Id', 'Sample','RF Level DAC','Chi-Squared',
                        'Run Duration (s)', 'Start Time','Title','Year',
                        'Cryo Lift Set (mm)', 'Cryo Lift Read (mm)',
                        'He Mass Flow','CryoEx Mass Flow', 'Needle Set (turns)',
                        'Needle Read (turns)','Laser Power', 'Target Bias (kV)',
-                       'NBM Rate (count/s)', 'Sample Rate (count/s)']
+                       'NBM Rate (count/s)', 'Sample Rate (count/s)')
     try: 
         bnmr_data_dir = os.environ[bnmr_archive_label]
         bnqr_data_dir = os.environ[bnqr_archive_label]
@@ -345,10 +355,7 @@ class bfit(object):
                       'elinewidth':mpl.rcParams['lines.linewidth'],
                       'alpha':1.,
                       'fillstyle':'full'}
-        
-        # load default fitting routines
-        self.routine_mod = importlib.import_module(
-                                                'bfit.fitting.default_routines')
+                                                
         # main frame
         mainframe = ttk.Frame(root,pad=5)
         mainframe.grid(column=0,row=0,sticky=(N,W,E,S))
@@ -386,8 +393,6 @@ class bfit(object):
         menu_settings.add_cascade(menu=menu_settings_dir,label='Data directory')
         menu_settings.add_command(label='Drawing style',
                 command=self.set_draw_style)
-        menu_settings.add_command(label='Fitting routines',
-                command=self.set_fit_routines)
         menu_settings.add_command(label='Histograms',
                 command=self.set_histograms)
         menu_settings.add_cascade(menu=menu_settings_lab,label='Labels default')                
@@ -469,6 +474,22 @@ class bfit(object):
         menu_draw.add_checkbutton(label="Use NBM",\
                 variable=self.use_nbm,selectcolor=colors.selected)
         
+        # Fitting minimizers
+        menu_mini = Menu(menubar)
+        menubar.add_cascade(menu=menu_mini, label='Minimizer')
+        
+        self.minimizer = StringVar()
+        self.minimizer.set(list(self.minimizers.values())[0])
+        for k,m in self.minimizers.items():
+            menu_mini.add_radiobutton(label=k,\
+                    variable=self.minimizer,
+                    value=m,
+                    selectcolor=colors.selected,
+                    command=self.set_fit_routine)
+        menu_mini.add_checkbutton(label='Other',\
+                variable=self.minimizer,
+                selectcolor=colors.selected,
+                command=self.set_fit_routine_with_popup)
         # Help
         menu_help = Menu(menubar)
         menubar.add_cascade(menu=menu_help, label='Help')
@@ -476,6 +497,9 @@ class bfit(object):
         menu_help.add_command(label="What's new?",command=self.whatsnew)
         menu_help.add_command(label='Update bfit',command=self.update_bfit)
         menu_help.add_command(label='Report an issue',command=self.report_issue)
+
+        # load default fitting routines
+        self.routine_mod = importlib.import_module(self.minimizer.get())
         
         # Top Notebook: File Viewer, Fit, Fit Viewer -------------------------
         noteframe = ttk.Frame(mainframe,relief='sunken',pad=5)
@@ -1310,7 +1334,7 @@ class bfit(object):
     def get_run_key(self,data=None,r=-1,y=-1):
         """For indexing data dictionary"""
         
-        if type(data) is bdata:
+        if type(data) in (bdata, fitdata):
             return '.'.join(map(str,(data.year,data.run)))
         elif type(data) is bmerged:
             runs = textwrap.wrap(str(data.run),5)
@@ -1413,7 +1437,23 @@ class bfit(object):
             self.logger.error('Input was not of type string')
     
     # ======================================================================= #
-    def set_fit_routines(self):
+    def set_fit_routine(self):
+        """Set python module for fitting routines"""
+        
+        self.logger.info('Loading module...')
+        self.routine_mod = importlib.import_module(self.minimizer.get())
+        
+        # repopuate fitter
+        self.logger.info('Repopulating fitter...')
+        self.fit_files.fitter = self.routine_mod.fitter(
+                                    keyfn = self.get_run_key,
+                                    probe_species = self.probe_species.get())
+        self.fit_files.fit_routine_label['text'] = self.fit_files.fitter.__name__
+        self.fit_files.populate()
+        self.logger.info('Success.')
+        
+    # ======================================================================= #
+    def set_fit_routine_with_popup(self):
         """Set python module for fitting routines"""
         self.logger.info('Setting fitting backend routine')
         d = filedialog.askopenfilename(initialdir = "./",
@@ -1434,21 +1474,13 @@ class bfit(object):
             pwd = os.getcwd()
             
             # load the module
-            self.logger.info('Loading module...')
             os.chdir(os.path.dirname(path))
-            self.routine_mod = importlib.import_module(os.path.splitext(
-                                                        os.path.basename(d))[0])
+            self.minimizer.set(os.path.splitext(os.path.basename(d))[0])
+            self.set_fit_routine()
             os.chdir(pwd)
             
-            # repopuate fitter
-            self.logger.info('Repopulating fitter...')
-            self.fit_files.fitter = self.routine_mod.fitter(
-                                        keyfn = self.get_run_key,
-                                        probe_species = self.probe_species.get())
-            self.fit_files.populate()
         else:
             self.logger.error('Input was not of type string.')    
-        self.logger.info('Success.')
         
     # ======================================================================= #
     def set_icon(self,window):
